/*******************************************************************************
  GPIO PLIB

  Company:
    Microchip Technology Inc.

  File Name:
    plib_gpio.h

  Summary:
    GPIO PLIB Header File

  Description:
    This library provides an interface to control and interact with Parallel
    Input/Output controller (GPIO) module.

*******************************************************************************/

/*******************************************************************************
* Copyright (C) 2019 Microchip Technology Inc. and its subsidiaries.
*
* Subject to your compliance with these terms, you may use Microchip software
* and any derivatives exclusively with Microchip products. It is your
* responsibility to comply with third party license terms applicable to your
* use of third party software (including open source software) that may
* accompany Microchip software.
*
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
* EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
* WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
* PARTICULAR PURPOSE.
*
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
* FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
* ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
* THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
*******************************************************************************/

#ifndef PLIB_GPIO_H
#define PLIB_GPIO_H

#include <device.h>
#include <stdint.h>
#include <stdbool.h>
#include <stddef.h>

// DOM-IGNORE-BEGIN
#ifdef __cplusplus  // Provide C++ Compatibility

    extern "C" {

#endif
// DOM-IGNORE-END

// *****************************************************************************
// *****************************************************************************
// Section: Data types and constants
// *****************************************************************************
// *****************************************************************************


/*** Macros for P_V pin ***/
#define P_V_Set()               (LATASET = (1<<0))
#define P_V_Clear()             (LATACLR = (1<<0))
#define P_V_Toggle()            (LATAINV= (1<<0))
#define P_V_OutputEnable()      (TRISACLR = (1<<0))
#define P_V_InputEnable()       (TRISASET = (1<<0))
#define P_V_Get()               ((PORTA >> 0) & 0x1)
#define P_V_PIN                  GPIO_PIN_RA0

/*** Macros for LED1 pin ***/
#define LED1_Set()               (LATBSET = (1<<2))
#define LED1_Clear()             (LATBCLR = (1<<2))
#define LED1_Toggle()            (LATBINV= (1<<2))
#define LED1_OutputEnable()      (TRISBCLR = (1<<2))
#define LED1_InputEnable()       (TRISBSET = (1<<2))
#define LED1_Get()               ((PORTB >> 2) & 0x1)
#define LED1_PIN                  GPIO_PIN_RB2

/*** Macros for VPP pin ***/
#define VPP_Set()               (LATBSET = (1<<5))
#define VPP_Clear()             (LATBCLR = (1<<5))
#define VPP_Toggle()            (LATBINV= (1<<5))
#define VPP_OutputEnable()      (TRISBCLR = (1<<5))
#define VPP_InputEnable()       (TRISBSET = (1<<5))
#define VPP_Get()               ((PORTB >> 5) & 0x1)
#define VPP_PIN                  GPIO_PIN_RB5

/*** Macros for SDOUT pin ***/
#define SDOUT_Set()               (LATBSET = (1<<7))
#define SDOUT_Clear()             (LATBCLR = (1<<7))
#define SDOUT_Toggle()            (LATBINV= (1<<7))
#define SDOUT_OutputEnable()      (TRISBCLR = (1<<7))
#define SDOUT_InputEnable()       (TRISBSET = (1<<7))
#define SDOUT_Get()               ((PORTB >> 7) & 0x1)
#define SDOUT_PIN                  GPIO_PIN_RB7

/*** Macros for STB pin ***/
#define STB_Set()               (LATBSET = (1<<8))
#define STB_Clear()             (LATBCLR = (1<<8))
#define STB_Toggle()            (LATBINV= (1<<8))
#define STB_OutputEnable()      (TRISBCLR = (1<<8))
#define STB_InputEnable()       (TRISBSET = (1<<8))
#define STB_Get()               ((PORTB >> 8) & 0x1)
#define STB_PIN                  GPIO_PIN_RB8

/*** Macros for SDIN pin ***/
#define SDIN_Set()               (LATBSET = (1<<9))
#define SDIN_Clear()             (LATBCLR = (1<<9))
#define SDIN_Toggle()            (LATBINV= (1<<9))
#define SDIN_OutputEnable()      (TRISBCLR = (1<<9))
#define SDIN_InputEnable()       (TRISBSET = (1<<9))
#define SDIN_Get()               ((PORTB >> 9) & 0x1)
#define SDIN_PIN                  GPIO_PIN_RB9

/*** Macros for CLK pin ***/
#define CLK_Set()               (LATBSET = (1<<13))
#define CLK_Clear()             (LATBCLR = (1<<13))
#define CLK_Toggle()            (LATBINV= (1<<13))
#define CLK_OutputEnable()      (TRISBCLR = (1<<13))
#define CLK_InputEnable()       (TRISBSET = (1<<13))
#define CLK_Get()               ((PORTB >> 13) & 0x1)
#define CLK_PIN                  GPIO_PIN_RB13

/*** Macros for ERASE pin ***/
#define ERASE_Set()               (LATBSET = (1<<14))
#define ERASE_Clear()             (LATBCLR = (1<<14))
#define ERASE_Toggle()            (LATBINV= (1<<14))
#define ERASE_OutputEnable()      (TRISBCLR = (1<<14))
#define ERASE_InputEnable()       (TRISBSET = (1<<14))
#define ERASE_Get()               ((PORTB >> 14) & 0x1)
#define ERASE_PIN                  GPIO_PIN_RB14

/*** Macros for DATA_SELECT pin ***/
#define DATA_SELECT_Set()               (LATBSET = (1<<15))
#define DATA_SELECT_Clear()             (LATBCLR = (1<<15))
#define DATA_SELECT_Toggle()            (LATBINV= (1<<15))
#define DATA_SELECT_OutputEnable()      (TRISBCLR = (1<<15))
#define DATA_SELECT_InputEnable()       (TRISBSET = (1<<15))
#define DATA_SELECT_Get()               ((PORTB >> 15) & 0x1)
#define DATA_SELECT_PIN                  GPIO_PIN_RB15


// *****************************************************************************
/* GPIO Port

  Summary:
    Identifies the available GPIO Ports.

  Description:
    This enumeration identifies the available GPIO Ports.

  Remarks:
    The caller should not rely on the specific numbers assigned to any of
    these values as they may change from one processor to the next.

    Not all ports are available on all devices.  Refer to the specific
    device data sheet to determine which ports are supported.
*/
#define    GPIO_PORT_A  (0U)
#define    GPIO_PORT_B  (1U)
typedef uint32_t GPIO_PORT;

// *****************************************************************************
/* GPIO Port Pins

  Summary:
    Identifies the available GPIO port pins.

  Description:
    This enumeration identifies the available GPIO port pins.

  Remarks:
    The caller should not rely on the specific numbers assigned to any of
    these values as they may change from one processor to the next.

    Not all pins are available on all devices.  Refer to the specific
    device data sheet to determine which pins are supported.
*/
#define    GPIO_PIN_RA0  (0U)
#define    GPIO_PIN_RA1  (1U)
#define    GPIO_PIN_RA2  (2U)
#define    GPIO_PIN_RA3  (3U)
#define    GPIO_PIN_RA4  (4U)
#define    GPIO_PIN_RB0  (16U)
#define    GPIO_PIN_RB1  (17U)
#define    GPIO_PIN_RB2  (18U)
#define    GPIO_PIN_RB3  (19U)
#define    GPIO_PIN_RB4  (20U)
#define    GPIO_PIN_RB5  (21U)
#define    GPIO_PIN_RB7  (23U)
#define    GPIO_PIN_RB8  (24U)
#define    GPIO_PIN_RB9  (25U)
#define    GPIO_PIN_RB10  (26U)
#define    GPIO_PIN_RB11  (27U)
#define    GPIO_PIN_RB13  (29U)
#define    GPIO_PIN_RB14  (30U)
#define    GPIO_PIN_RB15  (31U)

    /* This element should not be used in any of the GPIO APIs.
       It will be used by other modules or application to denote that none of the GPIO Pin is used */
#define    GPIO_PIN_NONE    (-1)

typedef uint32_t GPIO_PIN;


void GPIO_Initialize(void);

// *****************************************************************************
// *****************************************************************************
// Section: GPIO Functions which operates on multiple pins of a port
// *****************************************************************************
// *****************************************************************************

uint32_t GPIO_PortRead(GPIO_PORT port);

void GPIO_PortWrite(GPIO_PORT port, uint32_t mask, uint32_t value);

uint32_t GPIO_PortLatchRead ( GPIO_PORT port );

void GPIO_PortSet(GPIO_PORT port, uint32_t mask);

void GPIO_PortClear(GPIO_PORT port, uint32_t mask);

void GPIO_PortToggle(GPIO_PORT port, uint32_t mask);

void GPIO_PortInputEnable(GPIO_PORT port, uint32_t mask);

void GPIO_PortOutputEnable(GPIO_PORT port, uint32_t mask);

// *****************************************************************************
// *****************************************************************************
// Section: GPIO Functions which operates on one pin at a time
// *****************************************************************************
// *****************************************************************************

static inline void GPIO_PinWrite(GPIO_PIN pin, bool value)
{
    GPIO_PortWrite((GPIO_PORT)(pin>>4), (uint32_t)(0x1) << (pin & 0xFU), (uint32_t)(value) << (pin & 0xFU));
}

static inline bool GPIO_PinRead(GPIO_PIN pin)
{
    return (bool)(((GPIO_PortRead((GPIO_PORT)(pin>>4))) >> (pin & 0xFU)) & 0x1U);
}

static inline bool GPIO_PinLatchRead(GPIO_PIN pin)
{
    return (bool)((GPIO_PortLatchRead((GPIO_PORT)(pin>>4)) >> (pin & 0xFU)) & 0x1U);
}

static inline void GPIO_PinToggle(GPIO_PIN pin)
{
    GPIO_PortToggle((GPIO_PORT)(pin>>4), 0x1UL << (pin & 0xFU));
}

static inline void GPIO_PinSet(GPIO_PIN pin)
{
    GPIO_PortSet((GPIO_PORT)(pin>>4), 0x1UL << (pin & 0xFU));
}

static inline void GPIO_PinClear(GPIO_PIN pin)
{
    GPIO_PortClear((GPIO_PORT)(pin>>4), 0x1UL << (pin & 0xFU));
}

static inline void GPIO_PinInputEnable(GPIO_PIN pin)
{
    GPIO_PortInputEnable((GPIO_PORT)(pin>>4), 0x1UL << (pin & 0xFU));
}

static inline void GPIO_PinOutputEnable(GPIO_PIN pin)
{
    GPIO_PortOutputEnable((GPIO_PORT)(pin>>4), 0x1UL << (pin & 0xFU));
}


// DOM-IGNORE-BEGIN
#ifdef __cplusplus  // Provide C++ Compatibility

    }

#endif
// DOM-IGNORE-END
#endif // PLIB_GPIO_H
